@@ -21,6 +21,8 @@
 #include "application.h"
 #include "dialog_new_database.h"
 #include "dialog_progress_creating.h"
+#include "translation/dialog_change_language.h"
+#include "translation/window_translations.h"
 #include "utils.h"
 #include <libgnome/gnome-help.h> //For gnome_help_display
 #include "config.h" //For VERSION.
@@ -36,6 +38,7 @@ App_Glom::App_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml
   type_base(cobject, "Glom"),
   m_pBoxTop(0),
   m_pFrame(0),
+  m_window_translations(0),
   m_menu_tables_ui_merge_id(0),
   m_menu_reports_ui_merge_id(0)
 {
@@ -51,7 +54,11 @@ App_Glom::App_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml
 
 App_Glom::~App_Glom()
 {
-
+  if(m_window_translations)
+  {
+    m_pFrame->remove_view(m_window_translations);
+    delete m_window_translations;
+  }
 }
 
 bool App_Glom::init(const Glib::ustring& document_uri)
@@ -81,7 +88,7 @@ bool App_Glom::init(const Glib::ustring& document_uri)
     if(!test)
       return offer_new_or_existing();
   }
-  
+
   return true;
   //show_all();
 }
@@ -288,6 +295,14 @@ void App_Glom::init_menus()
   m_listDeveloperActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_layout));
 
+  action = Gtk::Action::create("GlomAction_Menu_Developer_ChangeLanguage", _("_Test Translation"));
+  m_listDeveloperActions.push_back(action);
+  m_refActionGroup_Others->add(action, sigc::mem_fun(*this, &App_Glom::on_menu_developer_changelanguage));
+
+  action = Gtk::Action::create("GlomAction_Menu_Developer_Translations", _("_Translations"));
+  m_listDeveloperActions.push_back(action);
+  m_refActionGroup_Others->add(action, sigc::mem_fun(*this, &App_Glom::on_menu_developer_translations));
+
   m_refUIManager->insert_action_group(m_refActionGroup_Others);
 
   //Build part of the menu structure, to be merged in by using the "Bakery_MenuPH_Others" placeholder:
@@ -320,6 +335,9 @@ void App_Glom::init_menus()
     "        <menuitem action='GlomAction_Menu_Developer_Layout' />"
     "        <menuitem action='GlomAction_Menu_Developer_Users' />"
     "        <menuitem action='GlomAction_Menu_Developer_Reports' />"
+    "        <separator />"
+    "        <menuitem action='GlomAction_Menu_Developer_Translations' />"
+    "        <menuitem action='GlomAction_Menu_Developer_ChangeLanguage' />"
     "      </menu>"
     "    </placeholder>"
     "  </menubar>"
@@ -553,7 +571,7 @@ void App_Glom::on_userlevel_changed(AppState::userlevels /* userlevel */)
 void App_Glom::update_userlevel_ui()
 {
   AppState::userlevels userlevel = get_userlevel();
-  
+
   //Disable/Enable developer actions:
   for(type_listActions::iterator iter = m_listDeveloperActions.begin(); iter != m_listDeveloperActions.end(); ++iter)
   {
@@ -833,7 +851,7 @@ bool App_Glom::recreate_database(bool& user_cancelled)
     {
       dialog_progress.reset(dialog_progress_temp); //The dialog will be deleted (and hidden) when this function returns.
 
-      dialog_progress->set_transient_for(*dialog_progress_temp);
+      dialog_progress->set_transient_for(*this);
       dialog_progress->show();
 
       //Ensure that the dialog is shown, instead of waiting for the application to be idle:
@@ -876,11 +894,11 @@ bool App_Glom::recreate_database(bool& user_cancelled)
   Document_Glom::type_listTableInfo tables = pDocument->get_tables();
   for(Document_Glom::type_listTableInfo::const_iterator iter = tables.begin(); iter != tables.end(); ++iter)
   {
-    const TableInfo& table_info = *iter;
+    sharedptr<const TableInfo> table_info = *iter;
 
     //Create SQL to describe all fields in this table:
     Glib::ustring sql_fields;
-    Document_Glom::type_vecFields fields = pDocument->get_table_fields(table_info.get_name());
+    Document_Glom::type_vecFields fields = pDocument->get_table_fields(table_info->get_name());
 
     dialog_progress->pulse();
     const bool table_creation_succeeded = m_pFrame->create_table(table_info, fields);
@@ -905,7 +923,7 @@ bool App_Glom::recreate_database(bool& user_cancelled)
 
       //Add any example data to the table:
       dialog_progress->pulse();
-      const bool table_insert_succeeded = m_pFrame->insert_example_data(table_info.get_name());
+      const bool table_insert_succeeded = m_pFrame->insert_example_data(table_info->get_name());
       if(!table_insert_succeeded)
       {
         g_warning("App_Glom::recreate_database(): INSERT of example data failed with the newly-created database.");
@@ -970,20 +988,20 @@ void App_Glom::fill_menu_tables()
   const Document_Glom::type_listTableInfo tables = document->get_tables();
   for(Document_Glom::type_listTableInfo::const_iterator iter = tables.begin(); iter != tables.end(); ++iter)
   {
-    const TableInfo& table_info = *iter;
-    if(!table_info.m_hidden)
+    sharedptr<const TableInfo> table_info = *iter;
+    if(!table_info->m_hidden)
     {
-      const Glib::ustring action_name = "NavTableAction_" + table_info.get_name();
+      const Glib::ustring action_name = "NavTableAction_" + table_info->get_name();
 
       ui_description += "<menuitem action='" + action_name + "' />";
 
-      Glib::RefPtr<Gtk::Action> refAction = Gtk::Action::create(action_name, table_info.get_title_or_name());
+      Glib::RefPtr<Gtk::Action> refAction = Gtk::Action::create(action_name, GlomUtils::string_escape_underscores(table_info->get_title_or_name()));
       m_refNavTablesActionGroup->add(refAction,
-        sigc::bind( sigc::mem_fun(*m_pFrame, &Frame_Glom::on_box_tables_selected), table_info.m_name) );
+        sigc::bind( sigc::mem_fun(*m_pFrame, &Frame_Glom::on_box_tables_selected), table_info->get_name()) );
 
       m_listNavTableActions.push_back(refAction);
 
-      //m_refUIManager->add_ui(merge_id, path, table_info.m_title, refAction, UI_MANAGER_MENUITEM);
+      //m_refUIManager->add_ui(merge_id, path, table_info->m_title, refAction, UI_MANAGER_MENUITEM);
     }
   }
 
@@ -1030,24 +1048,23 @@ void App_Glom::fill_menu_reports(const Glib::ustring& table_name)
   const Document_Glom::type_listReports tables = document->get_report_names(table_name);
   for(Document_Glom::type_listReports::const_iterator iter = tables.begin(); iter != tables.end(); ++iter)
   {
-    Report report;
-    bool found =  document->get_report(table_name, *iter, report);
-    if(found)
+    sharedptr<Report> report = document->get_report(table_name, *iter);
+    if(report)
     {
-      const Glib::ustring report_name = report.get_name();
+      const Glib::ustring report_name = report->get_name();
       if(!report_name.empty())
       {
         const Glib::ustring action_name = "NavReportAction_" + report_name;
 
         ui_description += "<menuitem action='" + action_name + "' />";
 
-        Glib::RefPtr<Gtk::Action> refAction = Gtk::Action::create(action_name, report.get_title_or_name());
+        Glib::RefPtr<Gtk::Action> refAction = Gtk::Action::create( action_name, GlomUtils::string_escape_underscores(report->get_title_or_name()));
         m_refNavReportsActionGroup->add(refAction,
-          sigc::bind( sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_report_selected), report.m_name) );
+          sigc::bind( sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_report_selected), report->get_name()) );
 
         m_listNavReportActions.push_back(refAction);
 
-        //m_refUIManager->add_ui(merge_id, path, table_info.m_title, refAction, UI_MANAGER_MENUITEM);
+        //m_refUIManager->add_ui(merge_id, path, table_info->m_title, refAction, UI_MANAGER_MENUITEM);
       }
     }
   }
@@ -1127,7 +1144,7 @@ void App_Glom::on_menu_file_save_as_example()
       Document_Glom::type_listTableInfo list_table_info = document->get_tables();
       for(Document_Glom::type_listTableInfo::const_iterator iter = list_table_info.begin(); iter != list_table_info.end(); ++iter)
       {
-        const Glib::ustring table_name = iter->get_name();
+        const Glib::ustring table_name = (*iter)->get_name();
 
         //const type_vecFields vec_fields = document->get_table_fields(table_name);
 
@@ -1179,3 +1196,66 @@ void App_Glom::on_menu_file_save_as_example()
   }
 }
 
+void App_Glom::on_menu_developer_changelanguage()
+{
+  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_change_language");
+  if(refXml)
+  {
+    Dialog_ChangeLanguage* dialog = 0;
+    refXml->get_widget_derived("dialog_change_language", dialog);
+    if(dialog)
+    {
+      dialog->set_transient_for(*this);
+      const int response = dialog->run();
+      dialog->hide();
+
+      if(response == Gtk::RESPONSE_OK)
+      {
+        TranslatableItem::set_current_locale(dialog->get_locale());
+
+       //Get the translations from the document (in Operator mode, we only load the necessary translations.)
+       //This also updates the UI, so we show all the translated titles:
+       get_document()->load();
+
+       m_pFrame->show_table_refresh(); //load() doesn't seem to refresh the view.
+      }
+
+      delete dialog;
+    }
+  }
+}
+
+void App_Glom::on_menu_developer_translations()
+{
+  if(!m_window_translations)
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_translations");
+    refXml->get_widget_derived("window_translations", m_window_translations);
+    if(m_window_translations)
+    {
+      m_pFrame->add_view(m_window_translations);
+      m_window_translations->set_transient_for(*this);
+      m_window_translations->set_document(static_cast<Document_Glom*>(m_pDocument));
+      m_window_translations->load_from_document();
+      m_window_translations->show();
+
+      m_window_translations->signal_hide().connect(sigc::mem_fun(*this, &App_Glom::on_window_translations_hide));
+    }
+  }
+  else
+  {
+    m_window_translations->show();
+    m_window_translations->load_from_document();
+  }
+}
+
+void App_Glom::on_window_translations_hide()
+{
+  if(m_window_translations)
+  {
+    m_pFrame->on_developer_dialog_hide();
+  }
+}
+
+
+
