@@ -147,7 +147,6 @@ Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glo
   return get_text_for_gda_value(glom_type, value, std::locale("") /* the user's current locale */, numeric_format); //Get the current locale.
 }
 
-
 Glib::ustring GlomConversions::get_text_for_gda_value(Field::glom_field_type glom_type, const Gnome::Gda::Value& value, const std::locale& locale, const NumericFormat& numeric_format, bool iso_format)
 {
   if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_NULL) //The type can be null for any of the actual field types.
@@ -296,7 +295,7 @@ Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type,
     gda_date.year = the_c_time.tm_year + 1900; //The C time starts at 1900.
     gda_date.month = the_c_time.tm_mon + 1; //The C month starts at 0.
     gda_date.day = the_c_time.tm_mday; //THe C mday starts at 1.
-    
+
     return Gnome::Gda::Value(gda_date);
   }
   else if(glom_type == Field::TYPE_TIME)
@@ -757,40 +756,40 @@ Glib::ustring GlomConversions::get_escaped_binary_data(guint8* buffer, size_t bu
   //g_warning("GlomConversions::get_escaped_binary_data: debug: buffer ");
   //for(int i = 0; i < 10; ++i)
   //  g_warning("%02X (%c), ", (guint8)buffer[i], buffer[i]);
-        
+
   //TODO: Performance: Preallocate a string of the appropriate size.
   //Use an output parameter instead of copying it during return.
-  
+
   Glib::ustring result;
-  
+
   /* This should be much faster, and more correct, but it gives an error - "unterminated quoted string at or near: "
   size_t escaped_length = 0;
   guchar* escaped =  Glom_PQescapeBytea(buffer, buffer_size, &escaped_length);
-  
+
   if(escaped && escaped_length)
   {
     result = std::string((char*)escaped, escaped_length);
     free(escaped);
   }
   */
-  
-  
+
+
   if(buffer && buffer_size)
   {
     guint8* buffer_end = buffer + buffer_size;
     char byte_as_octal[4]; //3 digits, and a null terminator
-    
+
     for(guint8* pos = buffer; pos < buffer_end; ++pos)
     {
       sprintf(byte_as_octal, "%03o", *pos); //Format as octal with 3 digits.
       byte_as_octal[3] = 0;
-      
+
 //      g_warning("byte=%d, as_hex=%s", *pos, byte_as_octal);      
 
-      result += Glib::ustring("\\\\") + byte_as_octal;
+      result += Glib::ustring("\\") + byte_as_octal; //Note that this string must be escaped once more to convert \ ("\\") into \\ ("\\\\").
     }
   }
- 
+
   return result;
 }
 
@@ -798,7 +797,7 @@ Gnome::Gda::Value GlomConversions::parse_escaped_binary_data(const Glib::ustring
 {
   //Hopefully we don't need to use this because Gda does it for us when we read a part of a "SELECT" result into a Gnome::Value.
   //TODO: Performance
-  
+
   Gnome::Gda::Value result;
   size_t buffer_binary_length = 0;
   guchar* buffer_binary =  Glom_PQunescapeBytea((guchar*)escaped_data.c_str(), &buffer_binary_length);
@@ -807,10 +806,23 @@ Gnome::Gda::Value GlomConversions::parse_escaped_binary_data(const Glib::ustring
     result.set(buffer_binary, buffer_binary_length);
     free(buffer_binary);
   }
-  
+
   return result;
 }
 
+Gnome::Gda::Value GlomConversions::convert_value(const Gnome::Gda::Value& value, Field::glom_field_type target_glom_type)
+{
+  const Field::glom_field_type source_glom_type = Field::get_glom_type_for_gda_type(value.get_value_type());
+  if(source_glom_type == target_glom_type)
+    return value; //No conversion necessary.
+  else
+  {
+    const Glib::ustring text = get_text_for_gda_value(source_glom_type, value);
+    bool test = false;
+    return parse_value(target_glom_type, text, test, true /* iso_format */);
+  }
 
+  return value;
+}
 
 
