@@ -25,6 +25,7 @@
 #include <gtkmm/main.h>
 #include <libgnome/gnome-init.h> // For gnome_program_init().
 #include <libgnomevfsmm/uri.h>
+#include <libgnomecanvasmm.h>
 #include <glibmm/i18n.h>
 
 
@@ -85,11 +86,14 @@ main(int argc, char* argv[])
   try
   {
     //Initialize gnome_program, so that we can use gnome_help_display().    
-    gnome_program_init (PACKAGE, VERSION, LIBGNOME_MODULE, argc, argv,
-                            GNOME_PROGRAM_STANDARD_PROPERTIES, 0);
-
+    gnome_program_init(PACKAGE_NAME, PACKAGE_VERSION, LIBGNOME_MODULE, argc, argv,
+		    GNOME_PARAM_HUMAN_READABLE_NAME, _("Glom"),
+		    GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
+    
+    
     Gtk::Main mainInstance(argc, argv, context);
     Bakery::init();
+    Gnome::Canvas::init();
 
     //Get command-line parameters, if any:
     Glib::ustring input_uri = group.m_arg_filename;
