@@ -40,16 +40,18 @@ public:
 
 protected:
 
-  virtual void add_group(const Gtk::TreeModel::iterator& parent, const LayoutGroup& group);
-  virtual void fill_group(const Gtk::TreeModel::iterator& iter, LayoutGroup& group);
+  virtual void add_group(const Gtk::TreeModel::iterator& parent, const sharedptr<const LayoutGroup>& group);
+  virtual void fill_group(const Gtk::TreeModel::iterator& iter, const sharedptr<LayoutGroup>& group);
 
   //Enable/disable buttons, depending on treeview selection:
   virtual void enable_buttons();
 
   virtual void save_to_document();
 
-  bool offer_relationship_list(Relationship& relationship);
-  bool offer_field_list(LayoutItem_Field& field);
+  sharedptr<Relationship> offer_relationship_list();
+  sharedptr<LayoutItem_Field> offer_field_list();
+  sharedptr<LayoutItem_Field> offer_field_list(const sharedptr<const LayoutItem_Field>& start_field);
+  sharedptr<LayoutItem_Field> offer_field_layout(const sharedptr<const LayoutItem_Field>& start_field);
   Gtk::TreeModel::iterator get_selected_group_parent() const;
 
   //signal handlers:
@@ -59,7 +61,8 @@ protected:
   virtual void on_button_field_add();  
   virtual void on_button_field_add_group();
   virtual void on_button_add_related();
-  virtual void on_button_field_edit();  
+  virtual void on_button_field_formatting();
+  virtual void on_button_field_edit();
   virtual void on_treeview_fields_selection_changed();
 
   virtual void on_cell_data_name(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter);
@@ -76,6 +79,7 @@ protected:
   Gtk::Button* m_button_field_add_group;
   Gtk::Button* m_button_add_related;
   Gtk::Button* m_button_field_delete;
+  Gtk::Button* m_button_field_formatting;
   Gtk::Button* m_button_field_edit;
   Gtk::Label* m_label_table_name;
 
