@@ -68,7 +68,7 @@ Dialog_Layout_List::Dialog_Layout_List(BaseObjectType* cobject, const Glib::RefP
     m_model_fields->signal_row_changed().connect( sigc::mem_fun(*this, &Dialog_Layout_List::on_treemodel_row_changed) );
   }
 
- 
+
   refGlade->get_widget("button_field_up", m_button_field_up);
   m_button_field_up->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_List::on_button_field_up) );
 
@@ -114,16 +114,16 @@ void Dialog_Layout_List::set_document(const Glib::ustring& layout, Document_Glom
 
     if(mapGroups.empty())
     {
-      LayoutGroup group;
-      group.set_name("main");
+      sharedptr<LayoutGroup> group = sharedptr<LayoutGroup>::create();
+      group->set_name("main");
 
       guint field_sequence = 1; //0 means no sequence
       for(type_vecLayoutFields::const_iterator iter = table_fields.begin(); iter != table_fields.end(); ++iter)
       {
-        LayoutItem_Field item = *(*iter);
-        item.m_sequence = field_sequence;
+        sharedptr<LayoutItem_Field> item = glom_sharedptr_clone(*iter);
+        item->m_sequence = field_sequence;
 
-        group.add_item(item, field_sequence);
+        group->add_item(item, field_sequence);
 
         ++field_sequence;
       }
@@ -139,20 +139,23 @@ void Dialog_Layout_List::set_document(const Glib::ustring& layout, Document_Glom
     guint field_sequence = 1; //0 means no sequence
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = mapGroups.begin(); iter != mapGroups.end(); ++iter)
     {
-      const LayoutGroup& group = iter->second;
+      sharedptr<const LayoutGroup> group = iter->second;
 
       //Add the group's fields:
-      LayoutGroup::type_map_const_items items = group.get_items();
+      LayoutGroup::type_map_const_items items = group->get_items();
       for(LayoutGroup::type_map_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
       {
-        const LayoutItem_Field* item = dynamic_cast<const LayoutItem_Field*>(iter->second); 
+        sharedptr<const LayoutItem_Field> item = sharedptr<const LayoutItem_Field>::cast_dynamic(iter->second);
+        if(item)
+        {
+          Gtk::TreeModel::iterator iterTree = m_model_fields->append();
+          Gtk::TreeModel::Row row = *iterTree;
 
-        Gtk::TreeModel::iterator iterTree = m_model_fields->append();
-        Gtk::TreeModel::Row row = *iterTree;
-
-        row[m_ColumnsFields.m_col_layout_item] = *item;
-        row[m_ColumnsFields.m_col_sequence] = field_sequence;
-        ++field_sequence;
+          sharedptr<LayoutItem_Field> newitem((LayoutItem_Field*)item->clone());
+          row[m_ColumnsFields.m_col_layout_item] = newitem;
+          row[m_ColumnsFields.m_col_sequence] = field_sequence;
+          ++field_sequence;
+        }
       }
     }
 
@@ -231,22 +234,22 @@ void Dialog_Layout_List::save_to_document()
     Document_Glom::type_mapLayoutGroupSequence mapGroups;
 
     //Add the fields to the one group:
-    LayoutGroup others;
-    others.set_name("main");
-    others.m_sequence = 1;
+    sharedptr<LayoutGroup> others = sharedptr<LayoutGroup>::create();
+    others->set_name("main");
+    others->m_sequence = 1;
 
     guint field_sequence = 1; //0 means no sequence
     for(Gtk::TreeModel::iterator iterFields = m_model_fields->children().begin(); iterFields != m_model_fields->children().end(); ++iterFields)
     {
       Gtk::TreeModel::Row row = *iterFields;
 
-      LayoutItem_Field item = row[m_ColumnsFields.m_col_layout_item];
-      const Glib::ustring field_name = item.get_name();
+      sharedptr<LayoutItem_Field> item = row[m_ColumnsFields.m_col_layout_item];
+      const Glib::ustring field_name = item->get_name();
       if(!field_name.empty())
       {
-        item.m_sequence = field_sequence;
+        item->m_sequence = field_sequence;
 
-        others.add_item(item, field_sequence); //Add it to the group:
+        others->add_item(item, field_sequence); //Add it to the group:
 
         ++field_sequence;
       }
@@ -284,25 +287,26 @@ void Dialog_Layout_List::on_button_add_field()
       if(response == Gtk::RESPONSE_OK)
       {
         //Get the chosen field:
-        LayoutItem_Field field;
-        dialog->get_field_chosen(field);
-
-        //Add the field details to the layout treeview:
-        Gtk::TreeModel::iterator iter =  m_model_fields->append();
-
-        if(iter)
+        sharedptr<LayoutItem_Field> field = dialog->get_field_chosen();
+        if(field)
         {
-          Gtk::TreeModel::Row row = *iter;
-          row[m_ColumnsFields.m_col_layout_item] = field;
+          //Add the field details to the layout treeview:
+          Gtk::TreeModel::iterator iter =  m_model_fields->append();
 
-          //Scroll to, and select, the new row:
-          Glib::RefPtr<Gtk::TreeView::Selection> refTreeSelection = m_treeview_fields->get_selection();
-          if(refTreeSelection)
-            refTreeSelection->select(iter);
+          if(iter)
+          {
+            Gtk::TreeModel::Row row = *iter;
+            row[m_ColumnsFields.m_col_layout_item] = field;
 
-          m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
+            //Scroll to, and select, the new row:
+            Glib::RefPtr<Gtk::TreeView::Selection> refTreeSelection = m_treeview_fields->get_selection();
+            if(refTreeSelection)
+              refTreeSelection->select(iter);
 
-          treeview_fill_sequences(m_model_fields, m_ColumnsFields.m_col_sequence); //The document should have checked this already, but it does not hurt to check again.
+            m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
+
+            treeview_fill_sequences(m_model_fields, m_ColumnsFields.m_col_sequence); //The document should have checked this already, but it does not hurt to check again.
+          }
         }
       }
     }
@@ -313,6 +317,8 @@ void Dialog_Layout_List::on_button_add_field()
   {
     std::cerr << ex.what() << std::endl;
   }
+
+  enable_buttons();
 }
 
 void Dialog_Layout_List::on_button_delete()
@@ -329,6 +335,8 @@ void Dialog_Layout_List::on_button_delete()
       m_modified = true;
     }
   }
+
+  enable_buttons();
 }
 
 
@@ -343,14 +351,14 @@ void Dialog_Layout_List::on_cell_data_name(Gtk::CellRenderer* renderer, const Gt
       Gtk::TreeModel::Row row = *iter;
 
       //Indicate that it's a field in another table.
-      const LayoutItem_Field item = row[m_ColumnsFields.m_col_layout_item]; //TODO_performance: Reduce copying.
+      sharedptr<LayoutItem_Field> item = row[m_ColumnsFields.m_col_layout_item]; //TODO_performance: Reduce copying.
 
       Glib::ustring markup;
 
-      if(item.get_has_relationship_name())
-        markup = item.get_relationship_name() + "::";
+      if(item->get_has_relationship_name())
+        markup = item->get_relationship_name() + "::";
 
-      markup += item.get_name();
+      markup += item->get_name();
 
       renderer_text->property_markup() = markup;
 
@@ -379,31 +387,32 @@ void Dialog_Layout_List::on_button_edit_field()
         if(iter)
         {
           Gtk::TreeModel::Row row = *iter;
-          const LayoutItem_Field& field = row[m_ColumnsFields.m_col_layout_item];
+          sharedptr<LayoutItem_Field> field = row[m_ColumnsFields.m_col_layout_item];
 
           dialog->set_document(get_document(), m_table_name, field);
           dialog->set_transient_for(*this);
-          int response = dialog->run();
+          const int response = dialog->run();
           if(response == Gtk::RESPONSE_OK)
           {
             //Get the chosen field:
-            LayoutItem_Field field;
-            dialog->get_field_chosen(field);
-  
-            //Set the field details in the layout treeview:
-  
-            row[m_ColumnsFields.m_col_layout_item] = field;
-  
-            //Scroll to, and select, the new row:
-            /*
-            Glib::RefPtr<Gtk::TreeView::Selection> refTreeSelection = m_treeview_fields->get_selection();
-            if(refTreeSelection)
-              refTreeSelection->select(iter);
-  
-            m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
-  
-            treeview_fill_sequences(m_model_fields, m_ColumnsFields.m_col_sequence); //The document should have checked this already, but it does not hurt to check again.
-            */
+            sharedptr<LayoutItem_Field> field = dialog->get_field_chosen();
+            if(field)
+            {
+              //Set the field details in the layout treeview:
+
+              row[m_ColumnsFields.m_col_layout_item] = field;
+
+              //Scroll to, and select, the new row:
+              /*
+              Glib::RefPtr<Gtk::TreeView::Selection> refTreeSelection = m_treeview_fields->get_selection();
+              if(refTreeSelection)
+                refTreeSelection->select(iter);
+
+              m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
+
+              treeview_fill_sequences(m_model_fields, m_ColumnsFields.m_col_sequence); //The document should have checked this already, but it does not hurt to check again.
+              */
+            }
           }
         }
       }
@@ -439,8 +448,8 @@ void Dialog_Layout_List::on_button_field_formatting()
         if(iter)
         {
           Gtk::TreeModel::Row row = *iter;
-          LayoutItem_Field field = row[m_ColumnsFields.m_col_layout_item];
-          g_warning("field_name = %s, %s", field.m_field.get_name().c_str(), field.get_name().c_str());
+          sharedptr<LayoutItem_Field> field = row[m_ColumnsFields.m_col_layout_item];
+          //g_warning("field_name = %s, %s", field.m_field.get_name().c_str(), field.get_name().c_str());
 
           dialog->set_field(field, m_table_name);
           dialog->set_transient_for(*this);
@@ -449,8 +458,8 @@ void Dialog_Layout_List::on_button_field_formatting()
           if(response == Gtk::RESPONSE_OK)
           {
             //Get the chosen field:
-            bool test = dialog->get_field_chosen(field);
-            if(test)
+            sharedptr<LayoutItem_Field> field = dialog->get_field_chosen();
+            if(field)
               row[m_ColumnsFields.m_col_layout_item] = field;
           }
         }
