@@ -31,12 +31,12 @@ Dialog_Fields::Dialog_Fields(BaseObjectType* cobject, const Glib::RefPtr<Gnome::
 {
   refGlade->get_widget_derived("vbox_placeholder", m_box);
 
-  m_label_frame->set_text( Bakery::App_Gtk::util_bold_message(_("Field Definitions")) );
-  m_label_frame->set_use_markup();
-  
+  m_label_frame->set_markup( Bakery::App_Gtk::util_bold_message(_("Field Definitions")) );
+
+
   //Fill composite view:
   add_view(m_box);
-  
+
   show_all_children();
 }
 
