@@ -21,7 +21,8 @@
 #include "dialog_relationships.h"
 #include "../box_db_table.h"
 //#include <libgnome/gnome-i18n.h>
-#include <libintl.h>
+#include <bakery/App/App_Gtk.h> //For util_bold_message().
+#include <glibmm/i18n.h>
 
 
 Dialog_Relationships::Dialog_Relationships(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
@@ -30,29 +31,35 @@ Dialog_Relationships::Dialog_Relationships(BaseObjectType* cobject, const Glib::
 {
   refGlade->get_widget_derived("vbox_placeholder", m_box);
 
-  m_label_frame->set_text( gettext("<b>Relationships</b>") );
-  m_label_frame->set_use_markup();
-  
+  m_label_frame->set_markup( Bakery::App_Gtk::util_bold_message(_("Relationships")) );
+
   //Fill composite view:
   add_view(m_box);
-  
+
   show_all_children();
 }
 
 Dialog_Relationships::~Dialog_Relationships()
 {
+  remove_view(m_box);
 }
 
-void Dialog_Relationships::init_db_details(const Glib::ustring& strTableName)
+/**
+ * 
+ * @param strTableName 
+ */
+bool Dialog_Relationships::init_db_details(const Glib::ustring& strTableName)
 {
   if(m_box)
   {
     m_box->load_from_document();
 
     Dialog_Design::init_db_details(strTableName);
-    
-    m_box->init_db_details(strTableName);
+
+    return m_box->init_db_details(strTableName);
   }
+
+  return false;
 }
 
 void Dialog_Relationships::on_hide()
