@@ -19,18 +19,18 @@
  */
 
 #include "box_data_list_find.h"
-#include <libintl.h>
+#include <glibmm/i18n.h>
 
 Box_Data_List_Find::Box_Data_List_Find()
 : m_HBox(false, 6)
 {
-  m_strHint = gettext("Enter the search criteria and click [Find]\n Glom will then change to Data mode to display the results.");
+  //m_strHint = _("Enter the search criteria and click [Find]\n Glom will then change to Data mode to display the results.");
 
   m_HBox.pack_end(m_Button_Find, Gtk::PACK_SHRINK);
   pack_start(m_HBox, Gtk::PACK_SHRINK);
 
   m_Button_Find.property_can_default() = true; //TODO: Make this a real method in gtkmm?
-  
+
   show_all_children();
 }
 
@@ -38,22 +38,27 @@ Box_Data_List_Find::~Box_Data_List_Find()
 {
 }
 
+void Box_Data_List_Find::create_layout()
+{
+  Box_Data_List::create_layout();
+}
 
-void Box_Data_List_Find::fill_from_database()
+bool Box_Data_List_Find::fill_from_database()
 {
   Bakery::BusyCursor(*get_app_window());
 
-  Box_DB_Table::fill_from_database();
+  bool result = Box_DB_Table::fill_from_database();
 
   m_AddDel.remove_all();
 
   //Field Names:
-  fill_column_titles();
+  create_layout();
 
-  type_vecFields listFieldsToShow = get_fields_to_show();
-  m_Fields = listFieldsToShow; //Store the fields for later:
+  m_FieldsShown = get_fields_to_show();
 
-  m_AddDel.add_item(Gnome::Gda::Value("find"));
+  //m_AddDel.add_item(Gnome::Gda::Value("find"));
+
+  return result;
 }
 
 void Box_Data_List_Find::on_adddel_user_changed(const Gtk::TreeModel::iterator& /* row */, guint /* col */)
@@ -66,4 +71,13 @@ Gtk::Widget* Box_Data_List_Find::get_default_button() //override
 {
   return &m_Button_Find;
 }
- 
+
+void Box_Data_List_Find::on_adddel_user_requested_delete(const Gtk::TreeModel::iterator& /* rowStart */, const Gtk::TreeModel::iterator&  /* rowEnd TODO */)
+{
+  //Ignore this, instead of updating the database, as the base class does.
+}
+
+void Box_Data_List_Find::on_adddel_user_added(const Gtk::TreeModel::iterator& /* row */, guint /* col_with_first_value */)
+{
+  //Ignore this, instead of updating the database, as the base class does.
+}
